# -*- coding: utf-8 -*-
import argparse
import sys

import time

import redis
from log4python.Log4python import log
import traceback

from config_flow_mis import redis_info

reload(sys)
logger = log("PriorityTasks")
sys.setdefaultencoding('utf8')


class PriorityTasks:
    '''
    ServerDemo:
        def init_worker(task):
            pass  # process task
        if __name__ == '__main__':
            task = PriorityTasks("Test")
            task.worker(init_worker)
    ClientDemo:
        # priority in ["high", "mid", "low"]
        PriorityTasks("Test").send_task(priority, task_params_json)
    '''
    def __init__(self, queue_key=None, work_status_queue_name="txt_redis_working_flag"):
        self.queue_key = queue_key
        self.work_status_queue_name = work_status_queue_name
        self.key_file_tasks = self.queue_key+"_%s"
        self.keys_file_tasks = [self.queue_key+"_high", self.queue_key+"_mid", self.queue_key+"_low"]
        self.redisCli = redis.StrictRedis(host=redis_info['host'], port=redis_info['port'], password=redis_info['password'], db=1)

    def get_tasks_by_priority(self):
        task = None
        for key_task in self.keys_file_tasks:
            line = self.redisCli.lpop(key_task)
            if line:
                logger.debug("Key:[%s]; line:[%s]" % (key_task, line))
                task = line
                break
        return task

    def send_task(self, priority, task_params):
        if not self.queue_key:
            tip = "The QueueKey is None, Nothing to Do."
            logger.error(tip)
            print(tip)
            return None

        queue_key = self.get_priority_queue(priority)
        self.redisCli.rpush(queue_key, task_params)

    def get_priority_queue(self, priority):
        key_process_priority = self.key_file_tasks % "high"
        if priority in ["high", "mid", "low"]:
            key_process_priority = self.key_file_tasks % priority
        else:
            logger.error("Log priority Level Not Found [%s]" % priority)
        return key_process_priority

    def worker(self, task_func):
        if not self.queue_key:
            tip = "The QueueKey is None, Nothing to Do."
            logger.error(tip)
            print(tip)
            return None

        work_status = False
        work_flag = self.redisCli.get(self.work_status_queue_name)
        if work_flag == "on":
            work_status = True
            logger.debug("InitWork: Status is ON!!! ")
        else:
            work_status = False
            logger.debug("InitWork: Status is OFF!!!  Nothing to do....")

        while True:
            work_flag = self.redisCli.get(self.work_status_queue_name)
            if work_flag != "on":
                if work_status:
                    logger.debug("Work Status Change to OFF!!! ")
                work_status = False
                time.sleep(1)
                continue
            else:
                if work_status is False:
                    logger.debug("Work Status Change to ON!!! ")
                work_status = True

            task = self.get_tasks_by_priority()
            if task is None:
                time.sleep(1)
                continue

            task_func(task)

if __name__ == '__main__':
    try:
        parser = argparse.ArgumentParser()
        parser.add_argument("logFile", type=str, help="specify the log file's path")
        args = parser.parse_args()
        print(args.logFile)
    except Exception, ex:
        logger.debug("Error: %s" % ex)
        logger.debug(traceback.format_exc())


# -*- coding: utf-8 -*-
import argparse
import json
import sys
from log4python.Log4python import log
from elasticsearch import Elasticsearch
import traceback
reload(sys)
logger = log("AgssQuery")
sys.setdefaultencoding('utf8')


class EsClient:
    def __init__(self, es_url):
        self.es_url = es_url  #
        self.scroll = "1m"
        self.es = self.es_connect()

    def es_connect(self):
        es = Elasticsearch(
                [
                    self.es_url
                ],
                verify_certs=False,
                timeout=60*60
        )
        return es

    def es_search(self, index_name, body_query):
        res = self.es.search(index=index_name, body=body_query)
        return res

    def es_search_scroll(self, index_name, body_query):
        result = []
        page = self.es.search(index=index_name, body=body_query, scroll=self.scroll)
        sid = page['_scroll_id']
        scroll_size = page['hits']['total']
        result.append(page)

        # Start scrolling
        while scroll_size > 0:
            logger.debug("Scrolling...")
            page = self.es.scroll(scroll_id=sid, scroll=self.scroll)
            # Update the scroll ID
            sid = page['_scroll_id']
            # Get the number of results that we returned in the last scroll
            scroll_size = len(page['hits']['hits'])
            logger.debug("scroll size: " + str(scroll_size))
            result.append(page)
        return result


class AgssQuery:
    def __init__(self, config, es_url):
        self.es_client = EsClient(es_url)
        self.config_audit = config

    @staticmethod
    def construct_query(config_sys):
        sys_name = config_sys['desc']
        rule = config_sys['rule']

        index = rule['es_index']
        json_search = rule['search']['json']
        action = rule['action']

        return sys_name, index, json_search, action

    def main_query(self):
        (sys_name, es_index_name, json_search, action) = self.construct_query(self.config_audit)
        # query
        logger.debug(u"SYS:[%s]; QueryJSON:%s" % (sys_name, json.dumps(json_search, ensure_ascii=False, default=lambda x: x.__dict__)))
        response = self.es_client.es_search(es_index_name, json_search)
        logger.debug(u"SYS:[%s]; Result:%s" % (sys_name, json.dumps(response, ensure_ascii=False, default=lambda x: x.__dict__)))
        # process
        ret_process = action(self.config_audit, response)

        return ret_process

if __name__ == '__main__':
    try:
        parser = argparse.ArgumentParser()
        parser.add_argument("logFile", type=str, help="specify the log file's path")
        args = parser.parse_args()
        print(args.logFile)
    except Exception, ex:
        logger.debug("Error: %s" % ex)
        logger.debug(traceback.format_exc())


# coding=utf-8
import os
import sys
from subprocess import check_call
if sys.hexversion > 0x03000000:
    import winreg
else:
    import _winreg as winreg


class Win32Env:
    """Utility class to get/set windows environment variable"""

    def __init__(self, scope):
        assert scope in ('user', 'system')
        self.scope = scope
        if scope == 'user':
            self.root = winreg.HKEY_CURRENT_USER
            self.subkey = 'Environment'
        else:
            self.root = winreg.HKEY_LOCAL_MACHINE
            self.subkey = r'SYSTEM\CurrentControlSet\Control\Session Manager\Environment'

    def getenv(self, name):
        key = winreg.OpenKey(self.root, self.subkey, 0, winreg.KEY_READ)
        try:
            value, _ = winreg.QueryValueEx(key, name)
        except WindowsError:
            value = ''
        return value

    def setenv(self, name, value):
        # Note: for 'system' scope, you must run this as Administrator
        key = winreg.OpenKey(self.root, self.subkey, 0, winreg.KEY_ALL_ACCESS)
        winreg.SetValueEx(key, name, 0, winreg.REG_EXPAND_SZ, value)
        winreg.CloseKey(key)
        # For some strange reason, calling SendMessage from the current process
        # doesn't propagate environment changes at all.
        # TODO: handle CalledProcessError (for assert)
        check_call('''\
"%s" -c "import win32api, win32con; assert win32api.SendMessage(win32con.HWND_BROADCAST, win32con.WM_SETTINGCHANGE, 0, 'Environment')"''' % sys.executable)


if __name__ == "__main__":
    # 程序将显示PATH变量的值，并设置用户的PATHONPATH变量的值。
    e1 = Win32Env(scope="system")
    print(e1.getenv('PATH'))
    e2 = Win32Env(scope="user")
    e2.setenv('PYTHONPATH', os.path.expanduser(r'~\mymodules'))



# -*- coding: utf-8 -*-
import argparse
import json
import pprint
import sys
from os.path import dirname, basename
import xmltodict
from log4python.Log4python import log
import traceback
from unipath import Path
from xml.dom.minidom import parseString

from toolkits.system.dict2xml import dicttoxml
from toolkits.system.shellHelper import exec_shell_with_pipe

reload(sys)
logger = log("PdiHelper")
sys.setdefaultencoding('utf8')


class PdiHelper:
    def __init__(self, pdi_exec):
        self.pdi_exec_path = pdi_exec  # "/services/pdi/data-integration/pan.sh"

    # 定义xml转json的函数
    def xmltojson(self, xmlstr):
        # parse是的xml解析器
        xmlparse = xmltodict.parse(xmlstr)
        # json库dumps()是将dict转化成json格式，loads()是将json转化成dict格式。
        # dumps()方法的ident=1，格式化json
        jsonstr = json.dumps(xmlparse, indent=1)
        return jsonstr

    # json转xml函数
    def jsontoxml(self, jsonstr):
        # xmltodict库的unparse()json转xml
        xmlstr = xmltodict.unparse(jsonstr)
        return xmlstr

    def load_xml(self, xml_config_file):
        fp = open(xml_config_file)
        lines = fp.readlines()
        fp.close()

        config = ""
        for item in lines:
            config += item.strip()
        return config

    def write_xml(self, new_config_file, config_xml):
        fp_out = open(new_config_file, "w+")
        fp_out.write(config_xml)
        fp_out.close()

    def covert_xml(self, xml_config_file, config_set):
        str_xml = self.load_xml(xml_config_file)

        config_json = self.xmltojson(str_xml)
        obj_json = json.loads(config_json)
        print(obj_json)
        my_item_func = lambda x: x
        str_xml = dicttoxml(obj_json, attr_type=False, root=False,
                            item_func=my_item_func, expand_list_item_name_flag=False)

        dom = parseString(str_xml)
        str_xml = dom.toprettyxml()

        self.write_xml("%s_new" % xml_config_file, str_xml)

    def covert_config(self, xml_config_file, new_config_file, config_set):
        '''<?xml version="1.0" encoding="utf-8"?>
    <student>
    <course>
    <name>math</name>
    <score>90</score>
    </course><info>
    <name>name</name>
    <sex>male</sex>
    </info>
    <stid>10213</stid>
    <title>1</title>
    <title>2</title>
    <list>
    <hop>
    <a>a</a>
    </hop>
    <hop>
    <a>b</a>
    </hop>
    </list>
    </student>
        # {u'student': {u'info': {u'name': u'name', u'sex': u'male'}, u'course': {u'score': u'90', u'name': u'math'},
        #  u'stid': u'10213', u'list': {u'hop': [{u'a': u'a'}, {u'a': u'b'}]}, u'title': [u'1', u'2']}}
        '''
        ret = False
        str_xml = self.load_xml(xml_config_file)

        config_json = self.xmltojson(str_xml)
        obj_config = json.loads(config_json)
        if self.config_merge(obj_config, config_set):
            my_item_func = lambda x: x
            str_xml = dicttoxml(obj_config, attr_type=False, root=False,
                                item_func=my_item_func, expand_list_item_name_flag=False)
            dom = parseString(str_xml)
            str_xml = dom.toprettyxml()
            self.write_xml(new_config_file, str_xml.replace("&quot;", "\""))
            ret = True
        return ret

    def set_dict(self, dst_dict, key_list, val):
        if len(key_list) > 0:
            key = key_list[0]
            del key_list[0]
            # dst_dict[key] = set_dict_val(dst_dict[key], key_list, val)

            # logger.debug("Key:[%s]" % key)
            pos_key = key.find("[")
            if pos_key >= 0:
                key_search = key[:pos_key]
                if type(dst_dict) is list:
                    key_found = False
                    for item in dst_dict:
                        if item[key_search] == key[pos_key+1: -1]:
                            item = self.set_dict(item, key_list, val)
                            key_found = True
                    if not key_found:
                        logger.debug("Error:key_search[%s] Not Found!!" % key)
                else:
                    logger.debug("key_search:[%s]" % key_search)
                    raise
            else:
                if dict(dst_dict).has_key(key):
                    dst_dict[key] = self.set_dict(dst_dict[key], key_list, val)
                else:
                    logger.debug("Error:key[%s] not found." % key)
            return dst_dict
        else:
            return val

    def config_merge(self, config_template, config_list):
        ret = False
        obj_data = None
        try:
            for config_item in config_list:
                key_list = config_item['key'].split(".")
                val_set = config_item['val']
                obj_data = config_template
                config_template = self.set_dict(config_template, key_list, val_set)
            ret = True
        except Exception, ex:
            pprint.pprint(obj_data)
            logger.debug("Ojbect_Data:%s" % pprint.pformat(obj_data))
            logger.debug("Error: %s" % ex)
            logger.debug(traceback.format_exc())
        finally:
            return ret

    def fetch_data_from_es(self, pdi_config_file, url, query, data_output_path):
        log_path = "%s/log/" % dirname(data_output_path)
        tmp_config_path = "%s/config_tmp/" % dirname(data_output_path)
        log_file_name = "%s/output_%s.log" % (log_path, basename(data_output_path))
        if not Path(log_path).exists():
            Path(log_path).mkdir(True)

        if not Path(tmp_config_path).exists():
            Path(tmp_config_path).mkdir(True)

        cmd_ret = False
        try:
            # modify the config
            config_set = [
                {
                    'key': "transformation.step.name[url_query].fields.field.name[url].nullif",
                    'val': url
                },
                {
                    'key': "transformation.step.name[url_query].fields.field.name[req].nullif",
                    'val': query.replace("\n", "")
                },
                {
                    'key': "transformation.step.name[out_put].file.name",
                    'val': data_output_path
                }]

            final_kettle_config = "%s/%s_new" % (tmp_config_path, basename(pdi_config_file))
            if self.covert_config(pdi_config_file, final_kettle_config, config_set):
                # run process
                cmd = "%s -file %s -logfile %s" % (self.pdi_exec_path, final_kettle_config, log_file_name)
                logger.debug("CMD:[%s]" % cmd)
                exec_shell_with_pipe(cmd)
                cmd_ret = True
        except Exception, ex:
            logger.debug("Error: %s" % ex)
            logger.debug(traceback.format_exc())
        finally:
            return cmd_ret

if __name__ == '__main__':
    try:
        parser = argparse.ArgumentParser()
        parser.add_argument("logFile", type=str, help="specify the log file's path")
        args = parser.parse_args()
        print(args.logFile)
    except Exception, ex:
        logger.debug("Error: %s" % ex)
        logger.debug(traceback.format_exc())



# -*- coding: utf-8 -*-
import pprint
import sys
from log4python.Log4python import log
reload(sys)
logger = log("MisUserActionEtl")
sys.setdefaultencoding('utf8')
import esm


def multi_search(search_list, search_string):
    index = esm.Index()
    for item in search_list:
        index.enter(item)
    index.fix()
    ret = index.query(search_string)
    return ret


if __name__ == '__main__':
    search_words = [u"宝马", u"马", u"奔驰", u"保时捷"]
    search_str = u"哎呀，今天在楼下看到了宝马，我老家倒是有养马的，以前的邻居有个奔驰，不对是保时捷，大爷的，都是马"
    pprint.pprint(multi_search(search_words, search_str))





#!/usr/bin/python
# -*- coding: utf-8 -*-
import json
import sys
import pprint
import traceback
from os.path import basename
from exchangelib import Configuration, Account, Credentials, DELEGATE, Message, Mailbox, FileAttachment
from log4python.Log4python import log

logger = log("SendMail")
reload(sys)
sys.setdefaultencoding('utf8')


class SendMail:
    def __init__(self, username, password, ews_url, primary_smtp_address):
        self.username = username
        self.password = password
        self.ews_url = ews_url
        self.primary_smtp_address = primary_smtp_address
        self.attach_file_list = []
        self.to_recipients = []
        self.cc_recipients = []
        self.bcc_recipients = []
        self.subject = None
        self.body = None
        self.ews_account = self.connect_mail_server()

    def connect_mail_server(self):
        credentials = Credentials(username=self.username, password=self.password)
        config = Configuration(service_endpoint=self.ews_url, credentials=credentials)
        account = Account(primary_smtp_address=self.primary_smtp_address, config=config,
                          autodiscover=False, access_type=DELEGATE)
        return account

    @staticmethod
    def read_file_content(file_path):
        fp = open(file_path, "r+")
        lines = fp.readlines()
        return "".join(lines)

    @staticmethod
    def list_update(list_container, new_data):
        if new_data is not None:
            if type(new_data).__name__ in ["str", "unicode"]:
                list_container.append(new_data)
            else:
                list_container.extend(new_data)
        return list_container

    def mail_receiver(self, to_recipients, cc_recipients=None, bcc_recipients=None):
        self.list_update(self.to_recipients, to_recipients)
        self.list_update(self.cc_recipients, cc_recipients)
        self.list_update(self.bcc_recipients, bcc_recipients)

    def mail_assemble(self, subject, body, attach_file_list=None):
        self.list_update(self.attach_file_list, attach_file_list)
        logger.debug("attach_files:%s" % json.dumps(self.attach_file_list))
        self.subject = subject
        self.body = body

    def mail_send(self, receive_mailer, subject, mail_body, attach_file=None):
        if self.to_recipients is None:
            return False

        self.mail_receiver(receive_mailer)
        if attach_file:
            self.mail_assemble(subject, mail_body, attach_file)
        else:
            self.mail_assemble(subject, mail_body)

        m = Message(
                account=self.ews_account,
                subject=self.subject,
                body=self.body,
                to_recipients=self.to_recipients,
                cc_recipients=self.cc_recipients,
                bcc_recipients=self.bcc_recipients
        )

        for item in self.attach_file_list:
            logger.debug("attach_file:%s" % str(item))
            m.attach(FileAttachment(name=basename(item), content=self.read_file_content(item)))
        m.send()
        return True


if __name__ == '__main__':
    try:
        username = "lijiayue"
        password = "please change to your mail info"
        ews_url = "https://mail.qq.com/EWS/Exchange.asmx"
        receive_mail = "lijiayue@qq.com"
        primary_smtp_address = "lijiayue@qq.com"

        csv_file = "%s/%s" % ("/var/log/", u"访问.csv")
        csv_detail_file = "/var/log/test.txt"

        mail = SendMail(username, password, ews_url, primary_smtp_address)
        mail.mail_send(receive_mail, u"发现违规访问用户", u"\n日期：%s; 人员数量：%s；\n访问，见附件：%s\n详细访问信息，见附件：%s\n\n",
                       [csv_file, csv_detail_file])
    except Exception, ex:
        pprint.pprint("Error: %s" % ex)
        pprint.pprint(traceback.format_exc())



# -*- coding: utf-8 -*-
import argparse
import os
import re
import sys
import traceback
import subprocess
import arrow
import thread
from threading import Timer

import time
from log4python.Log4python import log
from multiprocessing import Process, current_process

reload(sys)
logger = log("utils_time")
sys.setdefaultencoding('utf8')


def run_with_timeout(timeout, default, f, *args, **kwargs):
    if not timeout:
        return f(*args, **kwargs)
    timeout_timer = Timer(timeout, thread.interrupt_main)
    try:
        timeout_timer.start()
        result = f(*args, **kwargs)
        return result
    except KeyboardInterrupt:
        return default
    finally:
        timeout_timer.cancel()


def exec_cmd(cmd, work_path):
    exec_shell_with_pipe(cmd, work_path=work_path)


def worker(cmd, work_path):
    p = Process(target=exec_cmd, args=(cmd, work_path))
    p.start()
    os._exit(1)


def exec_external_cmd_background(cmd, work_path=""):
    p = Process(target=worker, args=(cmd, work_path))
    p.start()
    p.join()


def file_is_used(monitor_file):
    # fuser or lsof to check file's status
    cmd = "lsof %s" % monitor_file
    ret = exec_shell_with_pipe(cmd)
    if not ret:
        return True
    else:
        return False


def exec_shell_with_pipe(cmd, timeout=0, work_path=""):
    """exeShellWithPipe("grep 'processor' /proc/cpuinfo | sort -u | wc -l")
    :param work_path:
    :param timeout:
    :param cmd:  exec command
    """
    result = []
    none_num = 0
    if cmd == "" or cmd is None:
        return "No Cmd Input"
    if work_path == "":
        scan_process = subprocess.Popen(cmd, shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
    else:
        scan_process = subprocess.Popen(cmd, shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE, cwd=work_path)
    while True:
        if none_num > 3:
            break
        if timeout != 0:
            ret = run_with_timeout(timeout, None, scan_process.stdout.readline)
        else:
            ret = scan_process.stdout.readline()
        if ret == "" or ret is None:
            none_num += 1
        else:
            result.append(ret.strip())
            none_num = 0
    return result


if __name__ == '__main__':
    try:
        parser = argparse.ArgumentParser()
        parser.add_argument("logFile", type=str, help="specify the log file's path")
        args = parser.parse_args()
        print(args.logFile)
    except Exception, ex:
        logger.debug("Error: %s" % ex)
        logger.debug(traceback.format_exc())


# -*- coding: utf-8 -*-
import argparse
import sys

import unicodecsv
from log4python.Log4python import log
import traceback
reload(sys)
logger = log("ExcelHelper")
sys.setdefaultencoding('utf8')


def write_cvs(column_names, rows, csv_file):
    try:
        with open(csv_file, 'wb') as f:
            # w = unicodecsv.writer(f, encoding='utf-8-sig')
            w = unicodecsv.writer(f, encoding='utf-8')
            w.writerow(column_names)
            w.writerows(rows)
        logger.debug("CsvFile:%s" % csv_file)
        ret = True
    except Exception, ex:
        ret = False
        logger.error(traceback.format_exc())
        logger.error('deal_msg error: %s' % ex)
    return ret


class ExcelHelper:
    def __init__(self, file_path, column_split=","):
        self.file_path = file_path
        self.column_split = column_split
        self.data = self.load_data()
        self.cloumns_names = []

    def load_data(self):
        fp = open(self.file_path)
        lines = fp.readlines()
        if len(lines) == 0:
            return []

        self.cloumns_names = lines[0].strip("\n\r").split(self.column_split)
        del lines[0]  # delete the title

        data = []
        for item in lines:
            try:
                columns_val = item.strip().split(self.column_split)
                tmp_row = {}
                for item_index in range(0, len(self.cloumns_names)):
                    if item_index >= len(columns_val):
                        val = ""
                    else:
                        val = columns_val[item_index]
                    tmp_row[self.cloumns_names[item_index]] = val
                data.append(tmp_row)
            except Exception, ex:
                logger.debug("Item:[%s]" % item)
                logger.error(traceback.format_exc())
                logger.error('deal_msg error: %s' % ex)
        return data

    def get_column_data(self, column_name):
        column_data = []
        for item in self.data:
            column_data.append(item[column_name])
        return column_data

    def get_dict_data(self, key_column_name, val_columns_name):
        data_dict = {}
        for item in self.data:
            tmp_val = {}
            for val_name in val_columns_name:
                tmp_val[val_name] = item[val_name]

            key_name = unicode(item[key_column_name])
            if dict(data_dict).has_key(key_name):
                data_dict[key_name].append(tmp_val)
            else:
                tmp_list = [tmp_val]
                data_dict[key_name] = tmp_list

        return data_dict

if __name__ == '__main__':
    try:
        parser = argparse.ArgumentParser()
        parser.add_argument("excelFile", type=str, help="specify the excel file's path")
        args = parser.parse_args()
        csv = ExcelHelper(args.excelFile)
    except Exception, ex:
        logger.debug("Error: %s" % ex)
        logger.debug(traceback.format_exc())


# -*- coding: utf-8 -*-
from functools import reduce
from collections import Iterator
from itertools import chain,groupby,product


class CollectionsHelper:
    def __init__(self, src=None):
        self.src=src

    def toList(self):
        return list(self.src)

    def toSet(self):
        return set(self.src)

    def toTuple(self):
        return tuple(self.src)

    def getSource(self):
        return self.src

    def map(self,func):
        return CollectionsHelper(map(func, self.src))

    def filter(self,predicate):
        return CollectionsHelper(filter(predicate, self.src))

    def reduce(self,func,identity=None):
        if identity is None:
            return reduce(func,self.src)
        else:
            return reduce(func,self.src,identity)

    def chain(self):
        return CollectionsHelper(chain.from_iterable(self.src))

    def groupby(self,func=None):
        return CollectionsHelper(map(lambda it:(it[0], list(it[1])), groupby(self.src, func)))

    def product(self,tag):
        return CollectionsHelper(product(self.src, tag))

    def all(self,predicate):
        return all(map(lambda it:predicate(it),self.src))

    def any(self,predicate):
        return any(map(lambda it:predicate(it),self.src))

    def first(self,predicate=None):
        if predicate is None:
            if isinstance(self.src,Iterator):
                return next(self.src)
            return next(iter(self.src))
        else :
            return next(filter(predicate,self.src))

    def firstOrNone(self,predicate=None):
        try:
            if predicate is None:
                if isinstance(self.src,Iterator):
                    return next(self.src)
                return next(iter(self.src))
            else :
                return next(filter(predicate,self.src))
        except StopIteration:
            return None


if __name__ == '__main__':
    testgroupdata=[{"id":1,"name":"wwb"},{"id":1,"name":"wxa"},{"id":1,"name":"wxb"},{"id":2,"name":"wxc"},{"id":2,"name":"wxd"}]
    print(CollectionsHelper(testgroupdata).groupby(lambda it:it['id']).toList())


# -*- coding: utf-8 -*-
import argparse
import sys
from log4python.Log4python import log
from simplecrypt import encrypt, decrypt
import traceback
from unipath import Path

reload(sys)
logger = log("MisUserActionEtl")
sys.setdefaultencoding('utf8')


def encrypt_file(file_name):
    ciphertext = encrypt('password', plaintext)


def decrypt_file(file_name):
    if not Path(file_name).exists():
        pass
    plaintext = decrypt('password', ciphertext)

if __name__ == '__main__':
    try:
        parser = argparse.ArgumentParser()
        parser.add_argument("action", type=str, help="action: [encrypt|decrypt]")
        parser.add_argument("file_path", type=str, help="action: [encrypt|decrypt]")
        args = parser.parse_args()

        if args.action not in ['encrypt', 'decrypt']:
            print("action value was wrong! please check.[encrypt|decrypt]")

        if args.action == 'encrypt':
            encrypt_file(args.file_path)
        elif args.action == 'decrypt':
            decrypt_file(args.file_path)
    except Exception, ex:
        logger.debug("Error: %s" % ex)
        logger.debug(traceback.format_exc())


# -*- coding: utf-8 -*-
import argparse
import os
import sys
import time
from os.path import dirname
from log4python.Log4python import log
import traceback
from unipath import Path

from toolkits.system.shellHelper import exec_shell_with_pipe, file_is_used

path_cur = os.path.dirname(os.path.realpath(__file__))
path_parent = "%s/../" % path_cur
sys.path.append(path_parent)
# from system.shellHelper import exec_shell_with_pipe, file_is_used

reload(sys)
logger = log("HiveCmd")
sys.setdefaultencoding('utf8')


class HiveCmd:
    def __init__(self, file_name, sql, db_name, yarn_queue):
        self.file_name = str(file_name).replace(":", "")
        self.sql = sql
        self.hive_hql_file = "%s.hql" % self.file_name
        self.hive_err_file = "%s.err" % self.file_name
        # "use sec_ods;\nset mapred.job.queue.name=root.anquanbu-yewuanquanbu.default; "
        self.hive_info = "use %s;\nset mapred.job.queue.name=%s; \n" % (db_name, yarn_queue)
        logger.debug("HiveCmd:filename[%s]; sql[%s]" % (self.file_name, self.sql))

        path_to_check = dirname(self.file_name)
        if not Path(path_to_check).exists():
            Path(path_to_check).mkdir(True)

    def query(self):
        hql_content = self.hive_info + "%s"
        hql_cmd = "nohup hive -f %s 2> %s > %s &"
        Path(self.hive_hql_file).write_file(hql_content % self.sql)
        cmd = hql_cmd % (self.hive_hql_file, self.hive_err_file, self.file_name)
        exec_shell_with_pipe(cmd)

    def query_is_finished(self):
        return file_is_used(self.file_name)


if __name__ == '__main__':
    try:
        parser = argparse.ArgumentParser()
        parser.add_argument("fileName", type=str, help="specify the sql output file's path")
        parser.add_argument("sql", type=str, help="specify the sql to query")
        args = parser.parse_args()

        hive_cmd = HiveCmd(args.fileName, args.sql, "sec_ods", "root.anquanbu-yewuanquanbu.default")
        hive_cmd.query()
        while True:
            if hive_cmd.query_is_finished():
                break
            else:
                time.sleep(1)
    except Exception, ex:
        logger.debug("Error: %s" % ex)
        logger.debug(traceback.format_exc())



# -*- coding: utf-8 -*-
import argparse
import sys
from log4python.Log4python import log
import traceback
reload(sys)
logger = log("MisUserActionEtl")
sys.setdefaultencoding('utf8')


def set_dict_val(dst_dict, key_list, val):
    if len(key_list) > 0:
        key = key_list[0]
        del key_list[0]
        dst_dict[key] = set_dict_val(dst_dict[key], key_list, val)
        return dst_dict
    else:
        return val


if __name__ == '__main__':
    try:
        parser = argparse.ArgumentParser()
        parser.add_argument("logFile", type=str, help="specify the log file's path")
        args = parser.parse_args()
        print(args.logFile)
    except Exception, ex:
        logger.debug("Error: %s" % ex)
        logger.debug(traceback.format_exc())


import socket
import struct

import sys
from log4python.Log4python import log
reload(sys)
sys.setdefaultencoding('utf8')
logger = log("IpHelper")


def map_strip(str):
    tmp = str.lstrip("0")
    if tmp == "":
        tmp = "0"
    return tmp


def ip2long(ipstr):
    split_str = ipstr.split(".")
    split_str_map = map(map_strip, split_str)
    ip = ".".join(split_str_map)
    final_ip = 0
    try:
        final_ip = struct.unpack("!I", socket.inet_aton(ip))[0]
    except:
        logger.error("ErrIP: [%s]" % ipstr)
    return final_ip


def long2ip(ip):
    ip = int(ip)
    return socket.inet_ntoa(struct.pack("!I", ip))



# -*- coding: utf-8 -*-
import argparse
import re
import sys
import arrow
from log4python.Log4python import log
import traceback
reload(sys)
logger = log("timHelper")
sys.setdefaultencoding('utf8')


def get_arrow_time(time_str):
    ret_time = None
    try:
        ret = re.search("([0-9]{4})([0-9]{2})([0-9]{2})", time_str, re.MULTILINE)
        if not ret:
            ret = arrow.get(time_str)
            if ret:
                ret_time = ret.timestamp
        else:
            ret_time = arrow.get("%s-%s-%s 00:00:00" % (ret.group(1), ret.group(2), ret.group(3))).timestamp
    except Exception, ex:
        logger.debug("Error: %s" % ex)
        logger.debug(traceback.format_exc())
    finally:
        return ret_time


def get_month_end(timestamp):
    arr = arrow.get(timestamp)
    return arrow.get(arr.shift(months=1).format("YYYY-MM-01 00:00:00")).timestamp


def get_time_range_interval(time_begin, time_end, interval="1d"):
    ''' usage:
        time_day_begin = arrow.get(ret["time_begin"]).format('YYYY-MM-DD')
        time_day_end = arrow.get(ret["time_end"]).format('YYYY-MM-DD')'''
    ts_begin = get_arrow_time(time_begin)
    ts_end = get_arrow_time(time_end)
    ts_query_end = ts_begin

    time_range = []
    unit = interval[-1:]
    number = interval[:-1]
    secs_unit = {
                    "h": 60 * 60,
                    "m": 60,
                    "d": 60 * 60 * 24
                    }
    time_interval = 1

    if unit in secs_unit.keys():
        time_interval = secs_unit[unit] * int(number)
    else:
        logger.error("Err: interval format error[%s]!!" % interval)
        exit(-1)
    if ts_begin is None or ts_end is None:
        logger.error("time format error..")
        exit(-1)
    if ts_begin > ts_end:
        logger.error("Err: time_begin bigger than time_end!!")
        exit(-1)
    if ((ts_end - ts_begin) < time_interval) and (ts_end != ts_begin):
        logger.error("Err: 'interval' is smaller than the gap of between time_begin and time_end !!")
        exit(-1)

    while True:
        ts_query_begin = ts_query_end
        ts_query_end = ts_query_begin + time_interval
        ts_month_end = get_month_end(ts_query_begin)

        if ts_query_end > ts_month_end:
            if ts_query_end > ts_end:
                ts_query_end = ts_end
            else:
                ts_query_end = ts_month_end
        else:
            if ts_query_end > ts_end:
                ts_query_end = ts_end

        time_range.append({
            "time_begin": ts_query_begin,
            "time_end": ts_query_end
        })

        if ts_query_end >= ts_end:
            break
    return time_range


def get_time_range(time_begin, time_end, days_interval):
    ts_begin = get_arrow_time(time_begin)
    ts_end = get_arrow_time(time_end)
    time_range = []

    if ts_begin is None or ts_end is None:
        print("time format error..")
        exit(-1)
    ts_query_begin = ts_begin
    day_secs = 60 * 60 * 24
    ts_query_end = ts_query_begin - day_secs * 1

    while True:
        if ts_query_end >= ts_end:
            break

        time_interval = day_secs * 1
        if (ts_end - ts_begin) > (day_secs * days_interval):
            time_interval = day_secs * days_interval
        ts_query_begin = ts_query_end + day_secs * 1
        ts_query_end = ts_query_begin + time_interval
        ts_month_end = get_month_end(ts_query_begin)

        if ts_query_end > ts_month_end:
            if ts_query_end > ts_end:
                ts_query_end = ts_end
            else:
                ts_query_end = ts_month_end
        else:
            if ts_query_end > ts_end:
                ts_query_end = ts_end

        time_day_begin = arrow.get(ts_query_begin).format('YYYY-MM-DD')
        time_day_end = arrow.get(ts_query_end).format('YYYY-MM-DD')

        time_range.append({
            "time_begin": ts_query_begin,
            "time_end": ts_query_end
        })

    return time_range

if __name__ == '__main__':
    try:
        # parser = argparse.ArgumentParser()
        # parser.add_argument("logFile", type=str, help="specify the log file's path")
        # args = parser.parse_args()
        time_range = get_time_range_interval("2018-01-02 00:00:00", "2018-01-02 03:00:00", "1h")
        for time_item in time_range:
            print("TimeRange:%s-%s" % (arrow.get(time_item['time_begin']).format('YYYY-MM-DD HH:mm:ss'), arrow.get(time_item['time_end']).format('YYYY-MM-DD HH:mm:ss')))
    except Exception, ex:
        logger.debug("Error: %s" % ex)
        logger.debug(traceback.format_exc())



import sys
from distutils.core import setup
from distutils.command.install import INSTALL_SCHEMES

from setuptools import find_packages

PACKAGE = "toolkits"
NAME = "toolkits"
DESCRIPTION = "toolkits for quickly reference"
AUTHOR = "li_jia_yue"
AUTHOR_EMAIL = "59727816@qq.com"
URL = "https://github.com/starwithmoon/toolkits"
VERSION = __import__(PACKAGE).__version__

setup(
    name=NAME,
    version=VERSION,
    description=DESCRIPTION,
    long_description=open("README.rst").read(),
    author=AUTHOR,
    author_email=AUTHOR_EMAIL,
    license="BSD",
    url=URL,
    packages= find_packages(),  # [ PACKAGE, ],
    # packages=find_packages(exclude=["tests.*", "tests"]),
    # package_data=find_package_data(
	# 		PACKAGE,
	# 		only_in_packages=False
	#   ),
    keywords='toolkits tools',
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Natural Language :: English',
        "Programming Language :: Python :: 2",
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
    ],
    zip_safe=False,
)



# -*- coding: utf-8 -*-
import argparse
import pprint
import re
import sys

import sqlparse
from log4python.Log4python import log
from sqlparse.sql import Where, Comparison, Parenthesis, Identifier, Token, Function
import traceback

from sqlparse.tokens import DML

reload(sys)
logger = log("SqlHelper")
sys.setdefaultencoding('utf8')


class SqlWhere:
    def __init__(self, sql_dml):
        self.sql_dml = sql_dml
        self.where_tokens = None
        self.condition_data = {
            "condition": {
                "key": "",
                "op": "",
                "val": []
            },
            "or_list": [],
            "and_list": []
        }
        self.where_conditions = None

    @staticmethod
    def split_or(where_tokens):
        or_list = []
        or_item = []
        found_flag = False

        for item in where_tokens:
            if item.is_keyword and item.value.upper() == 'WHERE':
                continue

            if item.is_keyword and item.value.upper() == 'OR':
                found_flag = True
                # or_item_tmp = copy.deepcopy(or_item)
                or_list.append(or_item)
                or_item = []
            else:
                or_item.append(item)

        if found_flag:
            or_list.append(or_item)

        return or_list

    @staticmethod
    def split_and(where_tokens):
        and_list = []
        and_item = []
        found_flag = False
        between_flag = False

        for item in where_tokens:
            if item.is_keyword and item.value.upper() == 'WHERE':
                continue

            if item.is_keyword and item.value.upper() == 'BETWEEN':
                between_flag = True

            if item.is_keyword and item.value.upper() == 'AND' and not between_flag:
                found_flag = True
                and_list.append(and_item)
                and_item = []
            else:
                if item.is_keyword and item.value.upper() == 'AND' and between_flag:
                    between_flag = False
                and_item.append(item)

        if found_flag:
            and_list.append(and_item)

        return and_list

    @staticmethod
    def get_condition(tokens):
        key_found_flag = False
        val = []
        key = ""
        op = []

        for token in tokens:
            if token.is_whitespace:
                continue

            if token.is_keyword and token.value.upper() == 'WHERE':
                continue

            if isinstance(token, Comparison):
                key = token.left.value
                op = ["eq"]
                val = token.right.value
                break

            if token.is_keyword:
                if key_found_flag:
                    op.append(token.value)
            else:
                if not key_found_flag:
                    key = token.value
                    key_found_flag = True
                else:
                    if isinstance(token, Parenthesis):
                        tmp_val = token.value.strip("() ").split(",")
                        val_strip = [item.strip("\"") for item in tmp_val]
                        val.extend(val_strip)
                    else:
                        val.append(token.value)
                        # val += token.value
        condition_tmp = {
            "key": key,
            "op": "-".join(op).upper(),
            "val": val
        }

        return condition_tmp

    def check_sub_conditions(self, where_tokens):
        sub_tokens = []
        for token in where_tokens:
            if isinstance(token, Parenthesis) and (self.split_or(token.tokens) or self.split_and(token.tokens)):
                # sub_tokens.append(token.tokens)
                sub_tokens = token.tokens
        return sub_tokens

    def parse_where_tokens(self, where_tokens):
        where_group_tmp = {
            "condition": None,
            "sub_condition": None,
            "or_list": [],
            "and_list": [],
        }
        or_list = self.split_or(where_tokens)
        if or_list:
            for or_item in or_list:
                where_group_tmp['or_list'].append(self.parse_where_tokens(or_item))
        else:
            and_list = self.split_and(where_tokens)
            if and_list:
                for and_item in and_list:
                    where_group_tmp['and_list'].append(self.parse_where_tokens(and_item))
            else:
                sub_tokens = self.check_sub_conditions(where_tokens)
                if sub_tokens:
                    where_group_tmp['sub_condition'] = self.parse_where_tokens(sub_tokens)
                else:
                    where_group_tmp['condition'] = self.get_condition(where_tokens)

        return where_group_tmp

    def print_where(self):
        pprint.pprint(self.where_conditions)

    def get_column_condition(self, where_condition):
        column_condition = []
        if where_condition['condition']:
            column_condition.append(where_condition['condition'])

        if where_condition['sub_condition']:
            column_condition.extend(self.get_column_condition(where_condition['sub_condition']))

        if where_condition['or_list']:
            for item in where_condition['or_list']:
                column_condition.extend(self.get_column_condition(item))

        if where_condition['and_list']:
            for item in where_condition['and_list']:
                column_condition.extend(self.get_column_condition(item))

        return column_condition

    def get_where_conditions(self):
        where_tokens_list = self.get_sql_where()

        column_condition_list = []
        for where in where_tokens_list:
            tmp_where_tokens = self.parse_where_tokens(where)
            tmp_list = self.get_column_condition(tmp_where_tokens)
            column_condition_list.extend(tmp_list)

        return column_condition_list

    def has_dml(self, obj_check, level):
        sql_dml = None
        try:
            if level == 0:
                return sql_dml
            else:
                level -= 1
                found_dml = False
                if not isinstance(obj_check, Token):
                    for token_item in obj_check.tokens:
                        if token_item.ttype is DML:
                            found_dml = True
                            break
                if found_dml:
                    sql_dml = obj_check
                else:
                    if not isinstance(obj_check, Token):
                        for token_item in obj_check.tokens:
                            ret = self.has_dml(token_item, level)
                            if ret:
                                sql_dml = token_item
                                break
        except Exception, ex:
            sql_dml = None
            logger.debug("Error: %s" % ex)
            logger.debug(traceback.format_exc())
        finally:
            return sql_dml

    def get_where_tokens(self, tokens):
        tokens_where = []
        sql_dml = False
        for token_item in tokens:
            if token_item.ttype is DML:
                sql_dml = True
                break

        if not sql_dml:
            return tokens_where

        for token_item in tokens:
            if isinstance(token_item, Where):
                tokens_where.append(token_item)
            if isinstance(token_item, Identifier):
                token_check = self.has_dml(token_item, 2)
                if token_check:
                    sql_where = self.get_where_tokens(token_check.tokens)
                    if sql_where:
                        tokens_where.extend(sql_where)

        return tokens_where

    def search_columns(self, where_list, columns_list):
        for where_item in where_list:
            for token in where_item.tokens:
                for column in columns_list:
                    if token.is_group:
                        pass
                    elif token.is_keyword:
                        pass
                    elif token.is_whitespace:
                        pass
                    if isinstance(token, Identifier):
                        pass
                    if isinstance(token, Comparison):
                        pass
                    if isinstance(token, Function):
                        pass

    def get_sql_where(self):
        '''
        xx = "select * from test01 where id=1 or (id=2 and id=4) and id=5 or id = 6 and id not in (4,5,6)"
        xx = "select * from test01 where id=1 or (id=2 and id=4)"
        '''
        sql_query = str(re.sub("--.*?;", " ", self.sql_dml)).replace(";", " ")
        sql_list = sqlparse.parse(sql_query)
        where_tokens_list = []

        for sql in sql_list:
            where_list = re.findall("where", sql.value)
            where_num = len(where_list)
            if where_num == 0:
                print("NO found where.")
                break
            where_tokens = self.get_where_tokens(sql.tokens)
            where_tokens_list.extend(where_tokens)

        return where_tokens_list

if __name__ == '__main__':
    try:
        xx = "select * from test01 where id=1 or (id=2 and id=4)"
        sql_where = SqlWhere(xx)
        conditions_list = sql_where.get_where_conditions()
        pprint.pprint(conditions_list)
    except Exception, ex:
        logger.debug("Error: %s" % ex)
        logger.debug(traceback.format_exc())